// VinciaShower.cc is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// VINCIA is licenced under the GNU GPL version 2.
// Copyright (C) 2010 W.T. Giele, D.A. Kosower, P.Z. Skands.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// Function definitions (not found in the header) for the VinciaShower class, 
// as follows:
// 1) inherited member functions from the Pythia 8 TimeShower classs
// 2) non-inherited member functions for the VinciaShower class
// 3) member functions for the Branchelemental class

// Include the Vincia headers 
#include "VinciaShower.h"

using namespace Pythia8;

namespace Vincia {
  
  //**********************************************************************
  
  // The VinciaShower class and its members
  
  //**********************************************************************
  
  // BEGIN INHERITED FUNCTIONS FOR INTERFACE WITH PYTHIA8
  //  : init     : global shower initialization function
  //  : shower   : perform shower off partonic system
  //  : prepare  : prepare partonic system for showering
  //  : update   : purely dummy
  //  : rescatterUpdate : purely dummy
  //  : rescatterPropagateRecoil : purely dummy
  //  : pTnext   : generate next scale in downwards evolution
  //  : branch   : perform a branching, store results in event record
  
  //*********
  
  // Initialize Shower
  
  void VinciaShower::init( BeamParticle* beamAPtrIn, 
			   BeamParticle* beamBPtrIn) {
    
    // Verbose output
    if (verbose >= 3) {
      printOut("VinciaShower::init","initializing...");
    }
    
    // Store input pointers for future use. 
    beamAPtr     = beamAPtrIn;
    beamBPtr     = beamBPtrIn;
    
    // Verbose level and general precision target
    verbose            = settingsPtr->mode("Vincia:verbose");
    
    // Default evolution parameters
    evolutionType      = settingsPtr->mode("Vincia:evolutionType");
    cutoffType         = settingsPtr->mode("Vincia:cutoffType");
    if (cutoffType == -1) {
      printOut("VinciaShower::init",
            "Note - using Pythia to set hadronization cutoff");    
      cutoffScale = 2.0 * settingsPtr->parm("TimeShower:pTmin");
      settingsPtr->parm("Vincia:cutoffScale",cutoffScale);
    } else {
      if (cutoffType == 0) cutoffType = evolutionType;
      if (evolutionType == 3 && cutoffType != evolutionType) {
	printOut("VinciaShower::init",
            "Warning - forced cutoffType = 1 to regulate energy ordering");
	cutoffType = 1;
      }
      cutoffScale        = settingsPtr->parm("Vincia:cutoffScale");
    }    

    // Number of active quark flavours
    nGluonToQuark      = settingsPtr->mode("Vincia:nGluonToQuark");

    // Decide whether to impose sector ordering or not
    sectorOrdering     = settingsPtr->flag("Vincia:sectorOrdering");

    // Uncertainty Bands
    uncertaintyBands   = settingsPtr->flag("Vincia:uncertaintyBands");

    // Uncertainty Bands not yet implemented for sector showers
    if (sectorOrdering && uncertaintyBands) {
      uncertaintyBands = false;
      settingsPtr->flag("Vincia:uncertaintyBands",false);
      if (verbose >= 2) printOut("VinciaShower::init",
	"uncertainty bands are switched off for sector showers.");
    }

    // Hyperjet compatibility mode on/off
    hyperjet           = settingsPtr->flag("Vincia:hyperjet");    

    // Initialize Evolution Variable
    RPtr->init();

    // ME Matching     
    matching           = settingsPtr->flag("Vincia:matching");
    matchingStrategy   = settingsPtr->mode("Vincia:matchingStrategy");

    // ME Matching not yet implemented for sector showers
    if (sectorOrdering && matching) {
      matching = false;
      settingsPtr->flag("Vincia:matching",false);
      if (verbose >= 1) printOut("VinciaShower::init",
	"matching is switched off for sector showers.");
    }

    // Force strategy = 1 for the time being
    if (matchingStrategy != 1) {
      matchingStrategy = 1;
      if (matching) {
	if (verbose >= 1) printOut("VinciaShower::init",
				   "matchingStrategy changed to 1.");
      }
    }
    maxMatchTree       = settingsPtr->mode("Vincia:maxMatchTree");
    matching           = matching && maxMatchTree >= 1;
    matchingScaleType  = settingsPtr->mode("Vincia:matchingScaleType");
    matchingScale      = settingsPtr->parm("Vincia:matchingScale");
    matchingScaleRatio = settingsPtr->parm("Vincia:matchingScaleRatio");
    headRoomFactor     = settingsPtr->parm("Vincia:headRoomFactor");
    if (! matching) headRoomFactor = 1.0;
    // So far, the only way to have large enough trials for sector antennae
    // is by having a lot of headroom on a global-type trial
    // Should add dedicated sector trials later if possible
    if (sectorOrdering) headRoomFactor *= 10;

    // Decide whether to force strong (evolution) ordering or not
    orderingMode    = settingsPtr->mode("Vincia:orderingMode");

    // For pT-evolution, ordering mode 3 == ordering mode 2
    if (evolutionType == 1 && orderingMode == 3) orderingMode = 2;
    
    // Internal Histograms
    if (verbose >= 2) initVinciaHistos();
    
    // alphaS
    alphaSvalue        = settingsPtr->parm("Vincia:alphaSvalue");
    alphaSorder        = settingsPtr->mode("Vincia:alphaSorder");
    alphaSscaleFactor  = settingsPtr->parm("Vincia:alphaSscaleFactor");
    alphaSmode         = settingsPtr->mode("Vincia:alphaSmode"); 
    alphaSMax          = settingsPtr->parm("Vincia:alphaSMax"); 

    // If forcing pT as renormalization scale, but pT used as evolution scale,
    // revert to QE as scale (increases efficiency)
    if (alphaSmode == 3 && evolutionType == 1) alphaSmode = 1;

    // Initialize alphaS
    alphaS.init(alphaSvalue, alphaSorder); 

    // Check freezeout scale
    alphaSMuMin = settingsPtr->parm("Vincia:alphaSMuMin"); 
    if (alphaSorder >= 1) {
      // Make sure smallest ren scale > Lambda3
      alphaSMuMin = max(alphaSMuMin,1.1*alphaS.Lambda3()); 
    }
    alphaSMax = min(alphaSMax,alphaS.alphaS(pow2(alphaSMuMin)));

    // Lambda for 5, 4 and 3 flavours.
    Lambda3flav        = alphaS.Lambda3(); 
    Lambda4flav        = alphaS.Lambda4(); 
    Lambda5flav        = alphaS.Lambda5(); 
    Lambda5flav2       = pow2(Lambda5flav);
    Lambda4flav2       = pow2(Lambda4flav);
    Lambda3flav2       = pow2(Lambda3flav);
     
    // Set absolute lower limit on evolution
    if (alphaSorder == 1) {
      QMinAll = 1.1*Lambda3flav/alphaSscaleFactor;      
    } else {
      QMinAll = 0.1; 
    }
    if (evolutionType == cutoffType || cutoffType == 0) {
      QMinAll     = max(QMinAll,cutoffScale);
    } 
    cutoffScale = max(cutoffScale,QMinAll);

    // Masses   
    mt          = particleDataPtr->mass(6);
    mtb         = sqrt(particleDataPtr->mass(5)*particleDataPtr->mass(6));
    mb          = particleDataPtr->mass(5);
    mc          = particleDataPtr->mass(4);
    
    // NLL Matching
    NLLMatchingLevel = settingsPtr->mode("Vincia:NLLMatchingLevel");
    if (NLLMatchingLevel >= 0) {
      // Switch off NLL if using constant alphaS
      if (alphaSorder <= 0) {
	if (verbose >= 1) printOut("VinciaShower::init",
  	   "Cannot do NLL with constant alphaS. NLLMatchingLevel changed to -1.");
	NLLMatchingLevel = -1;
      }
      // Check if headRoomFactor should be increased (max by factor 2)
      else {
	double NLLHeadroom = 1.0;
	if (alphaSscaleFactor > 0.5) 
	  NLLHeadroom *= 1. + log(alphaSscaleFactor/0.5);
	if (alphaSmode == 2) 
	  NLLHeadroom *= 1. + alphaSscaleFactor;	
	NLLHeadroom = min(2.,NLLHeadroom);
	headRoomFactor *= NLLHeadroom;
      }
    }

    // Flag whether to use scale() associated with particles or not
    useScale               = settingsPtr->flag("Vincia:useCreationScales");
    
    // Enhancement factor on scale choice
    pTmaxFudge  = settingsPtr->parm("Vincia:pTmaxFudge");
    
    // Initialize negative-weight and non-unity warning flags
    negativeWeightOccurred = false;
    nonunityWeightOccurred = false;
    setWeightAll(0.0);
    resetWeightSums(0.0);
    reweightMaxPSaveP      = 0.0;
    reweightMaxPSaveQ      = 0.0;
    reweightMaxQSaveP      = 0.0;
    reweightMaxQSaveQ      = 0.0;
    
    // Check if interface to MadGraph is available and switched on
    matchingPtr->init();
    isMGOn = settingsPtr->flag("Vincia:MGInterface");

    // Print VINCIA header and list of parameters
    if (verbose >= 1) header();    
    
  }
  
  //*********
  
  // Top-level routine to do a full time-like shower in resonance decay.
  
  int VinciaShower::shower(int iBeg, int iEnd, Event& event, double pTmax) {
    
    // Add new system, automatically with two empty beam slots.
    int iSys = partonSystemsPtr->addSys();
    
    // Verbose output
    if (verbose >= 8) 
      printOut("VinciaShower::shower","preparing to shower. System no. "
	     +num2str(iSys));
    
    // Loop over allowed range to find all final-state particles.
    Vec4 pSum;
    for (int i = iBeg; i <= iEnd; ++i) if (event[i].isFinal()) {
	partonSystemsPtr->addOut( iSys, i);
	pSum += event[i].p();
      }
    partonSystemsPtr->setSHat( iSys, pSum.m2Calc() );
    
    // Let prepare routine do the setup.    
    prepare(iSys, event);
    
    // Begin evolution down in pT from hard pT scale. 
    int nBranch = 0;
    do {
      double pTtimes = pTnext(event, pTmax, 0.);
      
      // Do a final-state emission (if allowed).
      if (pTtimes > 0.) {
	if (branch(event)) ++nBranch; 
	pTmax = pTtimes;
      }
      
      // Keep on evolving until nothing is left to be done.
      else pTmax = 0.;
    } while (pTmax > 0.); 
    
    // Return number of emissions that were performed.
    return nBranch;
    
  }
  
  //*********
  
  // Prepare system for evolution.
  
  void VinciaShower::prepare(int iSys, Event& event) {
    
    //If verbose, list event before showering
    if (verbose >= 4) {
      if (verbose >= 6 || iSys == 0) event.list();
      printOut("VinciaShower::prepare","preparing system no. "
	       +num2str(iSys)+" for showering. Size = "
	       +num2str(partonSystemsPtr->sizeOut(iSys)));
    }
    
    // Get generated event weight from Pythia.
    // Problem: for e+e- the first one is 1
    //          for dijets, the first one is 0. 
    //          how to tell the first time we are being called for an event?
    if (iSys <= 1) {
      setWeightAll(infoPtr->weight());
      for (int iSet=0; iSet<int(antennaSets.size()); iSet++) {
	weightOld[iSet] = 0.0;
      }
    }

    // Reset dipole list for first interaction and for resonance decays.
    int iInA = partonSystemsPtr->getInA(iSys);
    if (iSys == 0 || iInA == 0) {
      branchelementals.resize(0);
    }

    // Vector to find total invariant mass of system
    Vec4 pSum;

    // Find and save all FS colors and anticolors
    map<int,int> indexOfAcol;
    map<int,int> indexOfCol;

    // Loop over event record. Find indices of FS particles with color lines
    int sizeSystem = partonSystemsPtr->sizeOut(iSys);
    for (int i = 0; i < sizeSystem; ++i) {    
      int i1 = partonSystemsPtr->getOut( iSys, i);
      // Skip if not present in final state
      if ( !event[i1].isFinal() ) continue;
      int col= event[i1].col();
      int acol= event[i1].acol();
      if (col != 0) indexOfCol[col]=i1;
      if (acol != 0) indexOfAcol[acol]=i1;
      if (col != 0 || acol != 0) pSum += event[i1].p();
    }

    // Store total invariant mass and set initial unordering scale
    mSystem  = m(pSum);
    Qrestart = mSystem;
    Q2U1     = -1.;
    if (verbose >= 7) {
      printOut("VinciaShower::prepare",
	       "Total mass of showering system = "+num2str(mSystem));
    }

    // Now loop over colored particles 
    for (map<int,int>::iterator it=indexOfCol.begin(); 
	 it != indexOfCol.end(); ++it) {
      int col  = it->first;
      int i1   = it->second;
      int i2   = indexOfAcol[col];
      if (col == 0 || i1 == 0 || i2 == 0) continue;
      if (verbose >= 7) {
	printOut("VinciaShower::prepare",
		 "Creating antenna between "+num2str(i1,4)+", "
		 +num2str(i2,4)+": col = "+num2str(col,5)
		 +" ; scales = "+num2str(event[i1].scale())
		 +", "+num2str(event[i2].scale()));      
      }

      // Store dipole and set scale = invariant mass;
      Branchelemental trial(event,i1,i2,particleDataPtr);
      trial.old1=event[i1];
      trial.old2=event[i2];	        
      trial.system=iSys;

      // Save ancestor (for use in matching below)
      trial.iAncestor=event[i1].mother1();
      // If not from resonance decay, set mother to zero
      // (should be augmented to look further back?)
      if (event[i1].mother2() != 0 && event[i1].mother2() != trial.iAncestor)
	trial.iAncestor = 0;
      if (event[i2].mother1() != trial.iAncestor)
	trial.iAncestor = 0;
      if (event[i2].mother2() != 0 && event[i1].mother2() != trial.iAncestor)
	trial.iAncestor = 0;
      if (trial.iAncestor != 0) 
	if ( ! event[trial.iAncestor].isResonance() ) trial.iAncestor=0;
      if (trial.iAncestor != 0) {
	isResonance = true;
      }

      // Determine starting scale
      if ( ! useScale ) {
	trial.Qold=m(trial.old1,trial.old2);      
	if (!isResonance) {
	  if (event[trial.iAncestor].colType() != 0) {
	    Q2U1 = trial.Qold;
	    if (orderingMode >= 2) trial.unOrdering = true;
	  }
	}
      } else {
	// In VINCIA's normalization, multiply by 2
	trial.Qold=2.*min(trial.old1.scale(),trial.old2.scale());
	// If ancestor has color, use current scale as unordering scale
	if (!isResonance) {
	  if (event[trial.iAncestor].colType() != 0) {
	    Q2U1 = trial.Qold;
	    if (orderingMode >= 2) trial.unOrdering = true;
	  }
	}
      }
      trial.hasTrial=false;
      
      // Find neigbor on i1 side
      if (event[i1].acol() != 0) {
	int colL = event[i1].acol();
	trial.iL = indexOfCol[colL];	
	// Find next-to-nearest neigbor on i1 side
	if (trial.iL != 0 && event[trial.iL].acol() != 0) {
	  int colLL = event[trial.iL].acol();
	  trial.iLL = indexOfCol[colLL];	
	}
      }

      // Find neigbor on i2 side
      if (event[i2].col() != 0) {
	int colR = event[i2].col();
	trial.iR = indexOfAcol[colR];	
	// Find next-to-nearest neigbor on i1 side
	if (trial.iR != 0 && event[trial.iR].col() != 0) {
	  int colRR = event[trial.iR].col();
	  trial.iRR = indexOfAcol[colRR];	
	}
      }

      branchelementals.push_back(trial);
      
    }
    
    // Initialize nEmit and nSplit (include possible non-Born starting points)
    nEmit  = max(0,int(branchelementals.size())-1);
    nSplit = max(0,sizeSystem-2-nEmit);    

    // ME-MATCHING: first determine class of res-decay matrix element
    MEClass = 0;
    if (matching && int(branchelementals.size())>0 && isResonance) {
      int iRes    = branchelementals[0].iAncestor;
      int colType = event[iRes].colType();
      int chgType = event[iRes].chargeType();
      int spnType = event[iRes].spinType();
      // Colorless Resonances
      if (colType == 0) {
	// Colorless Vector Resonances
	if (spnType == 3) {
	  if (chgType == 0) MEClass = 23;  // Q=0, S=1, C=Singlet
	  else if (abs(chgType) == 1) MEClass = 24; // Q=1, S=1, C=Singlet	    
	}
      }
      // No other types implemented so far.
    }
    
    //If verbose output, list dipoles
    if (verbose >= 5) list();    
    
  }

//*********

// Select next pT in downwards evolution of the existing dipoles.

double VinciaShower::pTnext(Event& event, double pTbegAll, double pTendAll) {

  // Translate to VINCIA normalization: Q_max = s_max = 2pTevol_max
  double Qold = 2.*pTbegAll;

  // Initialize winner scale
  double Qwin = 0.;

  // Lowest phase space point
  double QMin = max(QMinAll,2.*pTendAll);

  // Verbose output
  if (verbose >= 4) {
    printOut("VinciaShower::pTnext","(re)starting evolution at scale Q = "
	     +num2str(Qold));
    if (verbose >= 6)  list();
  }

  // Loop over dipole-antennae
  for (int iDip = 0; iDip < int(branchelementals.size()); ++iDip) {
    
    // Shorthand for current dipole-antenna
    Branchelemental* trial=&branchelementals[iDip];
    int id1 = trial->old1.id();
    int id2 = trial->old2.id();

    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::pTnext","* processing antenna no"+num2str(iDip,4)
	       +" col = "+num2str(trial->old1.col()));
    }

    // Generate new trial branchings, starting from Qold
    double mDip = m(event[trial->iOld1],event[trial->iOld2]);
    double Qnew = min(Qold,mDip);
    if (trial->unOrdering) Qnew = min(mDip,Qrestart);

    // Check if any available phase space
    if (Qnew <= QMin) {
      if (verbose >= 7) {
	printOut("VinciaShower::pTnext","| Qbegin = "
		 +num2str(Qnew)+" < QMin = "+num2str(QMin)+". Qnew = 0.0");
      }
      Qnew = 0.0;
      // Set antenna as being ready for hadronization
      trial->hasTrial = true;
      trial->Qnew     = 0.0;
      trial->iNew     = 0;
    } 

    // Lookup saved Qnew if there is one
    else if (trial->hasTrial) {
      Qnew = trial->Qnew;
      if (trial->iNew == -1) Qnew = 0.0;
      if (verbose >= 8) {
	printOut("VinciaShower::pTnext","| Trial QE = "+num2str(Qnew)+
		 " retrieved");
      }
    }     
    
    // Generate new Qnew
    else {
      
      // Loop over evolution regions (nf, PS boundaries)
      bool acceptTrial      = false;
      while (not acceptTrial) {
	
	// Reset hasTrial
	trial->hasTrial      = false;

	// Determine region for alphaS running and Z overestimates
	// (when including massive quarks and actually matching 
	//  across flavor thresholds, we may have to be more careful)
	int nfNow            = getNf(Qnew);
	int nGluonToQuarkNow = min(nfNow,nGluonToQuark);
	double QMinNow       = getQMin(Qnew);
	
	// Verbose output
	if (verbose >= 7) {
	  printOut("VinciaShower::pTnext","| Qbegin = "+num2str(Qnew)+" QMinNow = "
		   +num2str(QMinNow)+" nfNow = "+num2str(nfNow));	
	}

	// Scaled Q2min
	double yEmin = pow2(QMinNow/mDip);
	
	// AlphaS factor
	double alphaFac = alphaSvalue;
	if (alphaSorder == 1) {	  
	  if (alphaSmode == 2) {
	    // Running alphaS outside integral.
	    alphaFac = alphaS.alphaS(pow2(alphaSscaleFactor*mDip));
	  } else if (alphaSmode == 3) {
	    // pT as scale with non-pT as evolution variable
	    // generate trials with constant alphaSMax
	    alphaFac = alphaSMax;
	  } else {
	    // Running alphaS inside integral; factor 1/b0.
	    alphaFac = 12.0*M_PI/(11.0*CA-2.0*nfNow);
	  }	  
	}

	// Factor 1 / 4pi from coupling / phase-space 
	alphaFac /= 4. * M_PI;
	
	// Z Boundaries and normalization factor for Z integrals: Gluon Emission
	double zMin  = 0.5;
	double zMax  = 0.5;
	double zNorm = 1.0; // zNorm = 2kQ
	if (evolutionType == 1) {
	  // pT-ordering (normalization 2*kQ = 1)
	  zMin  = 0.5*(1.0-sqrt(1.0-yEmin)); 
	  zMax  = 0.5*(1.0+sqrt(1.0-yEmin)); 
	} else if (evolutionType == 2) {
	  // mAnt-ordering (normalization 2*kQ = 2)
	  zMin  = yEmin/2.; 
	  zMax  = 1.-yEmin/2.;
	  zNorm = 2.0;
	} else if (evolutionType == 3) {
	  // Energy ordering (2*kQ = 1, z limits imposed by hadronization cut)
	  double yCut = pow2(cutoffScale/mDip);
	  if (cutoffType == 1) {
	    zMin  = 0.5*(1.0-sqrt(1.0-yCut));
	    zMax  = 0.5*(1.0+sqrt(1.0-yCut));
	  } else {
	    zMin  = yCut/2.;
	    zMax  = 1.-yCut/2.;
	  } 
	} else if (evolutionType == 4) {
	  // Anti-collinear (Upsilon) ordering (normalization 2*kQ = 4)
	  zMin  = 0.5*(1.-pow2(1.0-yEmin));
	  zMax  = 0.5*(1.+pow2(1.0-yEmin));
	  zNorm = 4.0;
	} else if (evolutionType == 5) {
	  // Collinear-safe energy ordering (2*kQ = 1, QE -> pT close to collinear)
	  zMin  = 0.5*(1.-pow(1.0-pow2(yEmin),0.25));
	  zMax  = 0.5*(1.+pow(1.0-pow2(yEmin),0.25));
	}
	double zIntEmit = zNorm * log(zMax*(1.-zMin)/zMin/(1.-zMax));
	// Color Factor * AlphaS/4pi * Z Integral
	// = 1/b for const aS or = 1/b' for QE-dep running aS.
	double NormEmit = CA * alphaFac * zIntEmit * headRoomFactor;
	
	// Add in gluon splitting, if allowed
	double NormSplit(0.0);
	if ( (id1 == 21 || id2 == 21) && nGluonToQuark >= 1) {
	  // For now, just use the same trial function as for emission / 4.
	  double zIntSplit = 0.5*zIntEmit;
	  NormSplit = nGluonToQuarkNow * TR * alphaFac * zIntSplit * headRoomFactor;
	  if (id1 == 21 && id2 == 21) NormSplit *= 2.; 
	}
	
	// Generate Qnew 
	if (alphaSorder == 0 || alphaSmode == 2 || alphaSmode == 3) {
	  // Running outside trial integral (extra 1/2 from square root)
	  Qnew = Qnew * pow(rndmPtr->flat(),1./(NormEmit+NormSplit)/2.);
	} else {
	  // Running inside integral 
	  double Lambda = Lambda5flav;
	  if (nfNow == 4) Lambda = Lambda4flav;
	  if (nfNow == 3) Lambda = Lambda3flav;
	  double expn = pow(rndmPtr->flat(),1./(NormEmit+NormSplit));
	  Qnew = Lambda/alphaSscaleFactor * pow(alphaSscaleFactor*Qnew/Lambda,expn);
	}
	
	// Check whether Region crossing occurred.
	if (Qnew >= QMinNow || QMinNow <= QMin) {

	  // Qnew generated inside present region or no new region to go to.
	  if (verbose >= 8) {
	    printOut("VinciaShower::pTnext","| trial QE = "+num2str(Qnew)+
		     " accepted with nf = "+num2str(nfNow));
	  }

	  // If below cutoff, store as zero, else store Qnew
	  if (Qnew < QMin) Qnew=0.0;
	  trial->Qnew     = Qnew;
	  trial->hasTrial = true;
	  acceptTrial = true;

	  // Select between gluon emission and gluon splitting
	  trial->iNew     = 0;
	  if (Qnew > 0. && NormSplit > 0.) {	    
	    double Ran = rndmPtr->flat()*(NormEmit + NormSplit) - NormEmit;
	    if (Ran > 0.) trial->iNew = 
			    int(nGluonToQuarkNow * Ran/NormSplit + 1); 
	  }

	} 

	// Crossed region, try again in next region
	else {

	  // Verbose Output
	  if (verbose >=8 ) {
	    printOut("VinciaShower::pTnext","=== Trial Rejected at QE = "
		     +num2str(Qnew)+" Crossed Region Boundary at QMinNow = "
		     +num2str(QMinNow)
		     +" (nf was "+num2str(nfNow)+")");
	  }

	  // Reset Qnew and force new trial generation
	  Qnew            = QMinNow;
	  acceptTrial     = false;

	}

	// Don't bother with accept/reject if Qnew already < TINY
	// (must be here, since a previous small Qnew could just be 
	//  a region crossing)
	if (Qnew < TINY) continue;

	// Freeze alphaS at alphaSMax (by veto if above alphaSMax)
	if (acceptTrial && alphaSorder == 1 && alphaSmode != 3) {
	  double mu2;
	  if (alphaSmode == 1) mu2 = pow2(Qnew*alphaSscaleFactor);
	  else mu2 = pow2(mDip*alphaSscaleFactor);
	  double aS = alphaS.alphaS(mu2); 
	  if (aS > alphaSMax) {
	    double wAlphaS = alphaSMax / aS;
	    if (rndmPtr->flat() > wAlphaS) {
	      acceptTrial     = false; 
	      if (verbose >= 8) {
		printOut("VinciaShower::pTnext",
			 "=== Trial Rejected at QE = "+num2str(Qnew)
			 +". alphaSTrial = "+num2str(aS)
			 +" > alphaSMax = "+num2str(alphaSMax));      
	      }
	    }
	  }
	}
      }
      
    }
      
    // Did I win?
    if (Qnew >= Qwin || Qwin <= 0.0) {      
      winner = trial;
      Qwin=Qnew;
    } 
    
    // Verbose output
    if (verbose >= 7) {                    
      printOut("VinciaShower::pTnext","> Qnew   = "
	       +num2str(Qnew)+"; type "+num2str(trial->iNew,3)
	       +(iDip+1 == int(branchelementals.size()) ?
		 "; final Qwin = ":"; current Qwin = ")
	       +num2str(Qwin));  
    }  
  }

  // Verbose output  
  if (verbose >= 5 && Qwin > 0.) {
    if (verbose >= 7)  list();
    printOut("VinciaShower::pTnext","=== Winner at scale Qnew = "+num2str(Qwin)
	     +" antenna type "+num2str(winner->iNew,3)
	     +"   col = "+num2str(winner->old1.col(),5));
  }

  // If winner was unordered type, update current restart scale for unordering
  if (winner->unOrdering) Qrestart = Qwin;

  // Test for end-of-shower condition
  if (Qwin <= QMin)  {

    // Verbose output
    if (verbose >= 5) {
      printOut("VinciaShower::pTnext","=== All trials now below cutoff.");
      Qwin = 0.0;
      if (verbose >= 6) {
	printOut("VinciaShower::pTnext","Final configuration was:");
	event.list();
      }
    }

    // The procedure below is slightly unstable.
    // Should find if it is possible to check whether a given system is
    // the last system to be considered and then update.

    // Add weight correction to cumulative sum (!= w0 if more than one system)
    double w0 = weight();
    weightSumSave[0] += w0 - weightOld[0];

    // For first system, add one to number of events. 
    if (weightOld[0] == 0.0) {
      nWeights++;
      nUnityWeight++;
    }

    // Check for nonunity weights
    if (abs(abs(w0)-1.0) > TINY && 
	(abs(abs(weightOld[0])-1.0) < TINY || weightOld[0] == 0)) {
	nNonunityWeight++;
	nUnityWeight--;
    }

    // Check for negative weights
    if (w0 < 0.0 && weightOld[0] >= 0.0) nNegativeWeight++;

    // Update old weight for the USR antennae
    weightOld[0] = w0;

    // Save uncertainty band weights
    if (uncertaintyBands) {
      for (int iSet=1; iSet<int(antennaSets.size()); iSet++) {
	// Add weiht correction to cumulative sum 
	weightSumSave[iSet] += weight(iSet) - weightOld[iSet];
	// Update old weights
	weightOld[iSet] = weight(iSet);
      }      
    }
  }

  // Return nonvanishing value if branching found
  // Translate back to PYTHIA normalization --> factor 2 (see above).
  // If unordered branching won, return input scale - this should make 
  // the overall evolution still proceed in an ordered way.
  return min(pTbegAll,max(0.0,Qwin/2.0)); 
  
}

  //*********
  
  bool VinciaShower::branch(Event& event, bool isInterleaved) {
    
    // Extract current QE and pT scales
    double Qnew  = winner->Qnew;
    double pT2New = winner->Qnew/4.;
    // Note: we do not yet have new1, new2, new3 !
    if (evolutionType != 1) {
      pT2New = RPtr->Q2E(winner->new1,winner->new2,winner->new3,1)/4.;
    }
    
    // Verbose output
    if (verbose >= 6) {
      printOut("VinciaShower::branch","* Processing Branching at scale Q = " 
	       +num2str(Qnew)+"   isInterleaved = "+bool2str(isInterleaved));
    }

    // Generate kinematics in CM frame. Return if failed.
    if (not generateKinematics(winner)) {
      winner->setTrial(false);    
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial failed kinematics. Returning."); 
      }
      return false;
    } else {
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial passed kinematics. Continuing...");
      }
    }
    
    // Check if branching should be vetoed 
    if (sectorOrdering && !sectorAccept(event,winner)) {
      winner->setTrial(false);
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial failed sector veto. Returning.");
      }
      return false;
    } else if (sectorOrdering) {
      if (verbose >= 7) {
	printOut("VinciaShower::branch",
		 "Trial passed sector veto. Continuing...");
      }
    }      
    
    // Only check end-of-shower condition for nominal ordering. 
    bool hadAccept = true;
    if (not hadAccept) {    
      winner->nFailed++;
      if (winner->nFailed < 50 && evolutionType != cutoffType) {
	winner->setTrial(false); 
      } else {
	// Switch off this branchelemental.
	winner->Qold = 0.0;
	winner->Qnew = 0.0;
      }
      if (verbose >= 5) {
	printOut("VinciaShower::branch"
		 ,"=== Trial failed hadronization veto. Returning.");
      }
      return false;
    } else {
      if (verbose >= 7) {
	printOut("VinciaShower::branch"
		 ,"Trial passed hadronization veto. Continuing...");
      }
    }
    
    // Find initial particles in dipole branching.
    int iOld1 = winner->iOld1;
    int iOld2 = winner->iOld2;
    int acol1 = event[iOld1].acol();
    int col2  = event[iOld2].col();
    iSysSel = winner->system;
    
    // If smooth ordering, check if branching should be vetoed
    // Power-suppressed unordering 
    double Pimp = 1.0;
    if (orderingMode >= 2 && winner->unOrdering) {
      // Compute 2*pT scale of new branching and compute Pimp factor
      double Q2New = pow2(Qnew);
      if (orderingMode == 2 && evolutionType != 1)  
	Q2New = RPtr->Q2E(winner->new1,winner->new2,winner->new3,1);
      Pimp = 1.0/(1.0 + Q2New/Q2U1);
      // Check for negative Pimp factor (normally should not happen!)
      if (Pimp < 0.0) {
	if (verbose >= 1) 
	  printErr("VinciaShower::branch","Pimp factor = "+num2str(Pimp)+" < 0");
	Pimp =0.0;
      }
      // Apply Pimp factor
      Paccept[0] *= Pimp;  
      if (uncertaintyBands) {
	for (int iSet=1; iSet<int(antennaSets.size()); iSet++) 
	  Paccept[iSet] *= Pimp;  
      }
      if (verbose >= 8) 
	cout<< " PS Pimp = "<<Pimp<<" -> Paccept[0] = "<<Paccept[0]<<endl;
    }
    
    // Matching: Which order are we currently at?
    int nEmitNow  = nEmit;
    int nSplitNow = nSplit;
    if (winner->new2.id() == 21) nEmitNow++; else nSplitNow++;
    bool doMatch = MEClass != 0 && nEmitNow + nSplitNow <= maxMatchTree;

    // For gluon splitting to quarks, impose additional improvements
    double Pari = 1.0;
    if (winner->iNew != 0) {

      // Impose ARIADNE factor for g->qqbar.
      //       Pari = 2/(1+sN/sP)    (sN = s(neighbor), sP = s(parent))
      // If s(neighbor) << s(parent) -> splitting suppressed (screened?)
      // If s(neighbor) >> s(parent) -> splitting enhanced (up to factor 2)      
      double sP = m2(event[iOld1],event[iOld2]);
      double sN;
      int iN;
      // Which side of dipole-antenna is splitting? 
      if (event[iOld1].id() != 21 || winner->new1.id() == 21) {
	iN = winner->iR;
	// Only apply if neighbor exists, is a quark, and is in final state 
	if (iN != 0 && event[iN].id() != 21 && event[iN].isFinal() ) {
	  sN = m2(event[iOld2],event[iN]);
	  Pari = 2.0*sN/(sP + sN);      
	}
      } else { 
	iN = winner->iL;
	// Only apply if neighbor exists, is a quark, and is in final state 
	if (iN != 0 && event[iN].id() != 21 && event[iN].isFinal() ) {
	  sN = m2(event[iN],event[iOld1]);	
	  Pari = 2.0*sN/(sP + sN);      
	}	  
      }
      // Check for negative Pari factor (normally should not happen!)
      if (Pimp < 0.0) {
	if (verbose >= 1) 
	  printErr("VinciaShower::branch","Pari factor = "+num2str(Pari)+" < 0");
	Pari = 0.0;
      }
      // Apply Pari factor
      Paccept[0] *= Pari;      
      if (uncertaintyBands) {
	for (int iSet=1; iSet<int(antennaSets.size()); iSet++) 
	  Paccept[iSet] *= Pari;      
      }

      if (verbose >= 8) 
	cout<< " PS Pari = "<<Pari<<" -> Paccept[0] = "<<Paccept[0]<<endl;
      //      cout<< "   Pari = "<<Pari<<" -> Paccept[0] = "<<Paccept[0]<<endl;
      
      // DEVELOPMENT, switched off in distribution version
      // Beyond matched orders: match to 2>4 splitting function
      // (probably needs to be refined to deal with a gluon 
      //  splitting immediately after last matched order ...)
      /*
      if (!doMatch) {
	vector<Vec4> p2to4, p2to3, pClu;
	int colTypeA, colTypeB;
	// Splitting gluon on RIGHT side of dipole-antenna
	if (event[iOld1].id() != 21 || winner->new1.id() == 21) {
	  p2to4.push_back(winner->new1.p());
	  p2to4.push_back(winner->new2.p());
	  p2to4.push_back(winner->new3.p());
	  p2to4.push_back(event[winner->iR].p());
	  colTypeA = winner->old1.colType();
	  colTypeB = event[winner->iR].colType();
	} 
	// Splitting gluon on LEFT side of dipole-antenna
	else { 
	  p2to4.push_back(event[winner->iL].p());
	  p2to4.push_back(winner->new1.p());
	  p2to4.push_back(winner->new2.p());
	  p2to4.push_back(winner->new3.p());
	  colTypeA = event[winner->iL].colType();
	  colTypeB = winner->old2.colType();
	}
	
	// Compute full 2>4 accept probability
	double P2to4 = 1.0;
	double s01   = m2(p2to4[0],p2to4[1]);
	double s12   = m2(p2to4[1],p2to4[2]);
	double s23   = m2(p2to4[2],p2to4[3]);
	double s012  = m2(p2to4[0],p2to4[1],p2to4[2]);
	double s123  = m2(p2to4[1],p2to4[2],p2to4[3]);
	double s4    = m2(p2to4[0],p2to4[1],p2to4[2],p2to4[3]);
	if (colTypeA == 1 && colTypeB == -1) {
	  // Gluon splitting inside a qqbar>qgqbar parent dipole-antenna
	  // Treat as non-identical quarks. Check possibility to include
	  // identical-quark interferences as well.
	  P2to4 = B40(p2to4,0,1,2,3);
	  // History A
	  int kinTypeA = antennae[iqgSplit].getKineMapType();
	  VinClu(kinTypeA,p2to4,0,1,2,pClu);
	  double sA01  = m2(pClu[0],pClu[1]);
	  double sA12  = m2(pClu[1],p2to4[3]);
	  double termA = antennae[iqqEmit].a2(sA01/s4,sA12/s4)/s4;
	  termA *= antennae[iqgSplit].a2(s01/s012,s12/s012)/s4;
	  // History B
	  int kinTypeB = antennae[iqgSplit].getKineMapType();
	  VinClu(kinTypeB,p2to4,1,2,3,pClu);
	  pClu.push_back(p2to4[3]);
	  double sB01  = m2(p2to4[0],pClu[0]);
	  double sB12  = m2(pClu[0],pClu[1]);
	  double termB = antennae[iqqEmit].a2(sB01/s4,sB12/s4)/s4;
	  termB *= antennae[iqgSplit].a2(s23/s123,s12/s123)/s4;
	} else if (colTypeA == 1 && colTypeB == 2) {
	  // Gluon splitting inside a qg>qgg parent dipole-antenna
	  // Check normalization, and possibility to add subleading-color term
	  //P2to4 = E40(p2to4,0,1,2,3);
	} else if (colTypeA == 2 && colTypeB == -1) {
	  // Gluon splitting inside a gqbar>ggqbar parent dipole-antenna
	  // Check normalization, and possibility to add subleading-color term
	  //P2to4 = E40(p2to4,3,2,1,0);
	} else if (colTypeA == 2 && colTypeB == 2) {
	  // Gluon splitting inside a gqbar>ggqbar parent dipole-antenna
	  // Check normalization, and possibility to add subleading-color term
	  //P2to4 = G40(p2to4,3,2,1,0);
	}
      }
      */
    }

    // Set mothers
    winner->new1.mothers(iOld1,iOld2);
    winner->new2.mothers(iOld1,iOld2);
    winner->new3.mothers(iOld1,iOld2);
    
    // Set colour flow, deleting branched dipoles and adding new ones.
    winner->new1.acol(acol1);
    winner->new1.col(0);
    winner->new2.acol(0);
    winner->new2.col(0);
    winner->new3.acol(0);
    winner->new3.col(col2);   
    if (winner->new1.colType() > 0) {          
      int col=event.nextColTag();   
      winner->new1.col(col);
      winner->new2.acol(col);
    } 
    if (winner->new2.colType() > 0) {
      int col=event.nextColTag();
      winner->new2.col(col);
      winner->new3.acol(col);
    }    

    // QCD order
    int nQCD = nEmitNow + nSplitNow;
	
    // ME Matching
    doMatch = doMatch && matchingPtr->isMatched(MEClass,nEmitNow,nSplitNow,isMGOn);

    // Decide whether to match over full phase space or impose matching scale
    bool imposeMatchingScale = false;
    // By default, matching scale imposed from 3rd order
    if (nQCD >= 3) imposeMatchingScale = true;
    // For additive matching, impose matching scale already from 2nd order
    else if (nQCD == 2 && matchingStrategy == 3) imposeMatchingScale = true;
    // Check pT scale vs ME matching scale
    double pT2Match=0.0;
    if (doMatch) {
      // If matchingScaleType is relative, determine matching scale
      if (matchingScaleType == 1) pT2Match = pow2(matchingScaleRatio * mSystem);
      else pT2Match = pow2(matchingScale);
      // Do not compute matching coefficient if already far below matching scale.
      if (imposeMatchingScale && pT2New < 0.1 * pT2Match) doMatch=false;
    }
      
    // Matrix-Element Matching
    vector<int> id;
    vector<Vec4> p;
    vector<double> PME;
    string HPME="none";
    if (doMatch && nQCD <= maxMatchTree && isMGOn) {      
      // TMP!
      //      Paccept[0] /= Pimp;
      if (verbose >= 8) 
	cout << "Testing Order "<<nEmitNow+nSplitNow<<" MG Matching ..." <<endl;
      double pT2NewMin = pT2New;
      int iAncestor = winner->iAncestor;
      id.push_back(event[iAncestor].id());
      p.push_back(event[iAncestor].p());
      for (int iDip=0;iDip<int(branchelementals.size());iDip++) {
	Branchelemental* prevAnt=&branchelementals[iDip];
	if (prevAnt->iOld1 == winner->iOld1 && 
	    prevAnt->iOld2 == winner->iOld2) {
	  id.push_back(winner->new1.id());
	  id.push_back(winner->new2.id());
	  id.push_back(winner->new3.id());
	  p.push_back(winner->new1.p());
	  p.push_back(winner->new2.p());
	  p.push_back(winner->new3.p());
	} else {
	  if (prevAnt->iOld1 != winner->iOld2) {	  
	    id.push_back(event[prevAnt->iOld1].id());
	    p.push_back(event[prevAnt->iOld1].p());
	  }
	  if (event[prevAnt->iOld2].colType() != 2) {
	    id.push_back(event[prevAnt->iOld2].id());
	    p.push_back(event[prevAnt->iOld2].p());
	  }
	}
      }

      // Compute matching scale dampening factor (if imposed)
      double DeltaDamp = 1.0;
      if (imposeMatchingScale) {	
	for (int ip=0; ip < int(p.size())-3; ip++) {
	  pT2NewMin = min(pT2NewMin,RPtr->Q2E(p[ip+1],p[ip+2],p[ip+3],1)/4.);
	}
	DeltaDamp = min(1.,exp(-(pT2Match/pT2NewMin+0.25)));
      }

      /*
      cout << " ids  =";
      for (int i=0;i<id.size();i++) cout<<" "<<id[i];      
      cout << " nMom = "<<p.size()<<endl;
      */
      // Get matching coefficients
      if (matchingPtr->getPME(1,id,p,PME)) {
	PME[0] = 1 + DeltaDamp * (PME[0] - 1.);
	Paccept[0] *= PME[0];    
	if (uncertaintyBands) {
	  for (int iSet=1; iSet<int(antennaSets.size()); iSet++) {
	    PME[iSet] = 1 + DeltaDamp * (PME[iSet] - 1.);
	    Paccept[iSet] *= PME[iSet];      	
	  }
	}
	// Fill diagnostics histos?
	if (verbose >= 2) {
	  string matchingHisto="1stOrderMatchingLog10";
	  if (nQCD == 2) matchingHisto="2ndOrderMatchingLog10";
	  else if (nQCD == 3) matchingHisto="3rdOrderMatchingLog10";
	  else if (nQCD == 4) matchingHisto="4thOrderMatchingLog10";
	  else if (nQCD == 5) matchingHisto="5thOrderMatchingLog10";
	  else if (nQCD == 6) matchingHisto="6thOrderMatchingLog10";
	  else if (nQCD == 7) matchingHisto="7thOrderMatchingLog10";
	  string HPacc = matchingHisto+"P(ME/TrialAnt)";
	  if (nSplitNow == nSplit) HPacc += ":Emit";
	  else HPacc += ":Split";
	  vinciaHistos[HPacc].fill(log10(max(TINY,Paccept[0])));	  	
	  HPME = matchingHisto+"P(ME/PhysAnt)";
	  if (nSplitNow == nSplit) HPME += ":Emit";
	  else HPME += ":Split";
	}
      }
    }
    
    // Accept/Reject Trial
    if (rndmPtr->flat() > Paccept[0]) {
      if (verbose >= 5) {
	printOut("::generateKinematics","=== Branching Vetoed. wPhys/wTrial = "
		 +num2str(Paccept[0]));
      }
      winner->setTrial(false);
      return false;    
    } else if (Paccept[0] > 1.0+TINY) {
      if (verbose >= 1) {      
	printErr("::generateKinematics",
		 "=== Branching P > 1 at QCD order "+num2str(nQCD,1)
		 +", Antenna = "+antennaSets[0]->getHumanName(winner->iAnt)
		 +", wPhys/wTrial = "+num2str(Paccept[0]));
	if (doMatch && verbose >= 8) cout<<" PME = "<<PME[0]<<endl; 
	if (verbose >= 5) winner->list();
	scaleWeight(Paccept[0],0);
	if (Paccept[0] > 1.0 && verbose >= 5) {
	  cout<<" Momentum Listing: "<<endl;
	  for (int ip=0; ip<int(p.size())-1; ip++) { 		    
	    cout<<" partons["<<ip<<"].px("<<p[ip+1].px()<<");"<<endl;
	    cout<<" partons["<<ip<<"].py("<<p[ip+1].py()<<");"<<endl;
	    cout<<" partons["<<ip<<"].pz("<<p[ip+1].pz()<<");"<<endl;
	    cout<<" partons["<<ip<<"].e("<<p[ip+1].e()<<");"<<endl;
	  }
	}

      }

      // Automatically increase headroom factor when violations encountered,
      // up to a maximum of a factor 10.
      if (headRoomFactor < 10.) headRoomFactor *= 1.1;
      
    }

    // Fill histo
    if (HPME != "none" && verbose >= 2) {
      vinciaHistos[HPME].fill(log10(max(TINY,PME[0])));	  	
    }

    // Rescale event weights for uncertainty band
    if (uncertaintyBands) {
      for (int iSet=1; iSet<int(antennaSets.size()); iSet++) 
	scaleWeight(Paccept[iSet],iSet);      	      
    }

    /*
    if (not matchingAccept(event,winner)) {
      winner->setTrial(false);
      printOut(6,"VinciaShower::branch","Trial failed reweighting");
      return false;
    }
    */

    // Put new particles into the event record.
    int iNew1 = event.append(winner->new1);
    int iNew2 = event.append(winner->new2);
    int iNew3 = event.append(winner->new3);

    // Mark original dipole partons as branched and set daughters/mothers.
    event[iOld1].statusNeg();
    event[iOld1].daughters(iNew1, iNew3); 
    event[iOld2].statusNeg();
    event[iOld2].daughters(iNew1, iNew3);
    
    // Finally update the list of systems.
    partonSystemsPtr->replace(iSysSel, iOld1, iNew1);  
    partonSystemsPtr->addOut(iSysSel, iNew2);
    partonSystemsPtr->replace(iSysSel, iOld2, iNew3); 
    
    // Update number of branchings
    nEmit  = nEmitNow;
    nSplit = nSplitNow;
    
    // Store old neighbours
    int iL  = winner->iL;
    int iLL = winner->iLL;
    int iR  = winner->iR;
    int iRR = winner->iRR;
    
    // Update old and add new dipoles
    if (verbose >= 5) {
      printOut("VinciaShower::branch",
	       "=== Branching Accepted. Updating dipole-antenna(e)");
    }
    if (winner->new1.colType() > 0) {        
      // Normal case, where iNew1-iNew2 has an LC color connection
      winner->reset(event,iNew1,iNew2,particleDataPtr);
      winner->old1 = event[iNew1];
      winner->old2 = event[iNew2];
      winner->Qold = min(winner->m(),Qnew);
      if (orderingMode == 0 || orderingMode >= 2) {
	winner->unOrdering = true;
      }
      // Update neighbours
      winner->iL=iL;
      winner->iLL=iLL;
      winner->iR=iNew3;
      winner->iRR=iR;

      // If second parton is octet, insert new antenna (iNew2,iNew3)
      if (event[iNew2].colType() > 0) {
	
	// Define new dipole-antenna
	Branchelemental trial(event,iNew2,iNew3,particleDataPtr);
	trial.system = iSysSel;
	trial.old1   = event[iNew2];
	trial.old2   = event[iNew3];	 
	trial.Qold   = min(trial.m(),Qnew);
	if (orderingMode == 0 || orderingMode >= 2) {
	  trial.unOrdering = true;
	}
	
	// Set tri- and quadrupole neighbors
	trial.iL       = iNew1;
	trial.iLL   = iL;
	trial.iR      = iR;
	trial.iRR = iRR;

	// Keep family name for matching
	trial.iAncestor=winner->iAncestor;
	
	// Add dipole-antenna to list, insert after iNew1-Inew2 to keep order
	for (int iDip=0; iDip<int(branchelementals.size()); iDip++) {
	  if (&branchelementals[iDip] == winner) {
	    branchelementals.insert(branchelementals.begin()+iDip+1,trial);      
	    break;
	  }
	}
	
      }
      
    } else {
      // Special case for qbar q-x final state, where only second pair is dipole
      winner->reset(event,iNew2,iNew3,particleDataPtr);
      winner->old1        = event[iNew2];
      winner->old2        = event[iNew3];
      winner->Qold        = min(winner->m(),Qnew);
      if (orderingMode == 0 || orderingMode >= 2) {
	winner->unOrdering     = true;
	Qrestart          = max(Qrestart,winner->m());
      }
      // Update neighbors
      winner->iL  = iNew1;
      winner->iLL = iL; 
      winner->iR  = iR;
      winner->iRR = iRR;
    }
    
    // Update neighbor pointers in other dipoles and unordering scales
    Qrestart         = mSystem;
    Q2U1             = pow2(Qrestart);
    for (int iDip = 0; iDip < int(branchelementals.size()); iDip++) {
      
      // Shorthand 
      Branchelemental* ant = &branchelementals[iDip];
      
      // If sector, restart everyone from CM
      if (sectorOrdering && (orderingMode == 0 || orderingMode >= 2)) {
	ant->setTrial(false);
	ant->Qold = max(Qnew,ant->m());
	ant->unOrdering = true;
      }

      // If unordering, restart everyone from CM
      if (orderingMode >= 2 || orderingMode == 0) ant->unOrdering = true;
      
      // Update current min(pT) scale for unordering
      int uoType = 1;
      if (orderingMode == 2) uoType = evolutionType;
      if (ant->iL != 0) 
	Q2U1 = min(Q2U1, RPtr->Q2E(event[ant->iL],event[ant->iOld1],event[ant->iOld2],uoType));
      if (ant->iR != 0) 
	Q2U1 = min(Q2U1,RPtr->Q2E(event[ant->iOld1],event[ant->iOld2],event[ant->iR],uoType));
      
      // The post-branching branchelementals
      if (ant->iOld1 == iNew1 || ant->iOld1 == iNew2) {
	// No update necessary for this pair (handled above)
	continue;
      }    
      
      // Next-to-next-to-nearest neighbour update
      if (ant->iLL == iOld2) branchelementals[iDip].iLL   = iNew3; 
      if (ant->iRR == iOld1) branchelementals[iDip].iRR = iNew1; 
      
      // Next-to-nearest neighbour update 
      if (branchelementals[iDip].iL  == iOld2) {
	branchelementals[iDip].iL       = iNew3; 
	branchelementals[iDip].iLL   = iNew2;
      }
      if (branchelementals[iDip].iR == iOld1) {
	branchelementals[iDip].iR      = iNew1; 
	branchelementals[iDip].iRR = iNew2; 
      }
      
      // Nearest neighbour update
      if (branchelementals[iDip].iOld1 == iOld2) {
	branchelementals[iDip].setTrial(false);
	branchelementals[iDip].nFailed     = 0;
	branchelementals[iDip].iOld1       = iNew3;
	branchelementals[iDip].old1        = event[iNew3];
	branchelementals[iDip].iL       = iNew2;
	branchelementals[iDip].iLL   = iNew1;
	branchelementals[iDip].Qold        = min(branchelementals[iDip].Qold, 
						 branchelementals[iDip].m());
      }
      if (branchelementals[iDip].iOld2 == iOld1) {
	branchelementals[iDip].setTrial(false);
	branchelementals[iDip].nFailed     = 0;	
	branchelementals[iDip].old2        = event[iNew1];
	branchelementals[iDip].iOld2       = iNew1;
	branchelementals[iDip].iR      = iNew2;
	branchelementals[iDip].iRR = iNew3;
	branchelementals[iDip].Qold        = min(branchelementals[iDip].Qold, 
						 branchelementals[iDip].m());
      }
      
    }
    
    if (verbose >= 6) {
      event.list();
      list();
    }
    
    // Done.      
    return true;
    
  }
  
  //*********
  
  // Update list of branchelementals after ISR radiation
  
  void VinciaShower::update( int iSys, Event& event) {
    if (verbose >= 1)
      printErr("VinciaShower::update","system "+num2str(iSys)
	       +" (function not implemented)");
    if (verbose >= 5) event.list();
  }
  
//*********
  
// Update list of branchelementals after MPI rescattering 
  
  void VinciaShower::rescatterUpdate( int iSys, Event& event) {
    if (verbose >= 1)
      printErr("VinciaShower::rescatterUpdate","system "+num2str(iSys)
	       +" (function not implemented)");
    if (verbose >= 5) event.list();
  }
  
  //*********
  
  // Update list of branchelementals after MPI rescattering 
  
  bool VinciaShower::rescatterPropagateRecoil( Event& event, Vec4& pNew) {
    if (verbose >= 1)
      printErr("VinciaShower::rescatterPropagateRecoil","function not implemented");
    if (verbose >= 5) event.list();
    if (verbose >= 9) printOut("VinciaShower::rescatterPropagateRecoil",
			       "pNew = "+num2str(pNew.px())+","+num2str(pNew.py())
			       +","+num2str(pNew.pz())+","+num2str(pNew.e()));
    return false;
  }

//**********************************************************************
  
// END OF INHERITED FUNCTIONS FOR INTERFACE TO PYTHIA8
// BEGIN VINCIA INTERNAL FUNCTIONS
  
//**********************************************************************

// VinciaShower class member functions

//*********

// Initialize internal and diagnostics histograms

void VinciaShower::initVinciaHistos() {

  vinciaHistos.clear();

  // Don't book diagnostics histograms if verbose output not switched on.
  if (verbose <= 1) return;

  // Global
  int nBin;
  double binWid, wid, c, xMin, xMax;

  // MATCHING WEIGHTS
  nBin   = 61;
  binWid = 0.1;
  wid    = nBin * binWid;
  c      = 0.0;
  xMin   = c - wid/2.;
  xMax   = c + wid/2.;
  Hist HPME2LogUSR("Matching weight (USR)",nBin,xMin,xMax);

  // Histos for Paccept : gluon emission
  vinciaHistos["1stOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/PhysAnt):Emit"]=HPME2LogUSR;

  // Histos for Paccept : gluon splitting
  vinciaHistos["1stOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/PhysAnt):Split"]=HPME2LogUSR;

  // Histos for PME : gluon emission
  vinciaHistos["1stOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/TrialAnt):Emit"]=HPME2LogUSR;

  // Histos for PME : gluon splitting
  vinciaHistos["1stOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["2ndOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["3rdOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["4thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["5thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["6thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;
  vinciaHistos["7thOrderMatchingLog10P(ME/TrialAnt):Split"]=HPME2LogUSR;

}

  //*********
  
  // Phase space regions
  
  double VinciaShower::getQMin(double Q) {
    double QMinNow;
    if (Q > mt) QMinNow = mt;
    else if (Q > mtb) QMinNow = mtb;
    else if (Q > mb) QMinNow = mb;
    else if (Q > mc) QMinNow = mc;
    else QMinNow = QMinAll;
    return QMinNow;
  }

  double VinciaShower::getQMin(int Nf) {
    if (Nf >= 6) return mt;
    else if (Nf == 5) return mb;
    else if (Nf == 4) return mc;
    else return QMinAll;
  }

  double VinciaShower::getQMax(int Nf) {
    if (Nf >= 6) return 1.e99;
    else if (Nf == 5) return mt;
    else if (Nf == 4) return mb;
    else return mc;
  }

  int VinciaShower::getNf(double Q) {
    if (Q > mt) return 6;
    else if (Q > mb) return 5;
    else if (Q > mc) return 4;
    else return 3;    
  }

  //*********
  
  // Definition of perturbative region : perturbativeTheta(a,1,b); a,1,b ordered.
  // Input: sred   = s - (m1+m2+m3)^2
  //        s12red = s12 - (m1+m2)^2
  //        s23red = s23 - (m2+m3)^2
  
  bool VinciaShower::perturbativeTheta(double s,double s12,double s23) {
    if (s <= 0.0) return false;
    else return RPtr->yE(s12/s,s23/s,cutoffType)*s > pow2(cutoffScale) ? 
	   true : false;}
  
  // Copy of perturbativeTheta using Branchelemental as input
  bool VinciaShower::perturbativeTheta(Branchelemental* trial) {
    double s12=trial->s12();
    double s23=trial->s23();
    double s=trial->s();
    return perturbativeTheta(s,s12,s23);
  }

  // Copy of perturbativeTheta using Vec4 as input
  bool VinciaShower::perturbativeTheta(Vec4 pa, Vec4 pr, Vec4 pb) {
    double sar=m2(pa+pr);
    double srb=m2(pr+pb);
    double s=m2(pa+pr+pb);
    return perturbativeTheta(s,sar,srb);
  }
  
//*********

// Generate full branching kinematics (assumes trial scale already set).
// Start construction in branching CM, then rotate and boost to LAB.

bool VinciaShower::generateKinematics(Branchelemental* trial) {

  // Verbose output
  if (verbose >= 8) {
    printOut("::generateKinematics","Generating trial kinematics ...");
  }

  // Global variables
  double Q2   = pow2(trial->Qnew);
  double s    = trial->s();  
  double mDip = sqrt(s);

  // Branching Type (emission = 0, splitting > 0)
  int branchType = trial->iNew;  

  // Consistency check 1
  if ( s < TINY || Q2 < TINY) {
    if (verbose >= 3) {
      printOut("::generateKinematics",
	       "Warning: s ="+num2str(s)+"  Q2 = "+num2str(Q2));
    }
    trial->nFailed++;
    if (trial->nFailed >= 50) {
      trial->Qnew = 0.0;
      trial->Qold = 0.0;
    }
    return false;
  }
  
  // Value of evolution variable and phase space boundary
  double yE    = Q2/s;
  double QMin  = getQMin(trial->Qnew);
  double yEmin = pow2(QMin)/s;

  // Z boundaries used for trial generation
  double zMax, zMin;
  if (evolutionType == 1) {
    // pT-ordering
    zMin = 0.5*(1.0-sqrt(1.0-yEmin)); 
    zMax = 0.5*(1.0+sqrt(1.0-yEmin)); 
  } else if (evolutionType == 2) {
    // Daughter-mass ordering
    zMin = yEmin/2.;
    zMax = 1.-yEmin/2.;
  } else if (evolutionType == 3) {
    // For energy ordering the z limits are those imposed by 
    // hadronization in Type 1 variable
    double yCut = pow2(cutoffScale/mDip);
    if (cutoffType == 1) {
      zMin = 0.5*(1.0-sqrt(1.0-yCut));
      zMax = 0.5*(1.0+sqrt(1.0-yCut));
    } else {
      zMin = yCut/2.;
      zMax = 1.-yCut/2.;
    }
  } else if (evolutionType == 4) {
    // Anti-collinear "V" ordering
    zMin  = 0.5*(1.-pow2(1.0-yEmin));
    zMax  = 0.5*(1.+pow2(1.0-yEmin));
  } else if (evolutionType == 5) {
    // Collinear-safe energy ordering (2*kQ = 1, QE -> pT close to collinear)
    zMin  = 0.5*(1.-pow(1.0-pow2(yEmin),0.25));
    zMax  = 0.5*(1.+pow(1.0-pow2(yEmin),0.25));
  } 
  // Use same as Type 1 as a catchall (should not occur)
  else {
    zMin = 0.5*(1.0-sqrt(1.0-yEmin)); 
    zMax = 0.5*(1.0+sqrt(1.0-yEmin)); 
  }
  
  // Select z
  double rMax = zMax/(1.-zMax);
  double rMin = zMin/(1.-zMin);
  double rNow = rMin * pow(rMax/rMin,rndmPtr->flat());
  double z = rNow/(1.+rNow);  

  // Derive y12, y23 from yE, z
  double y12, y23;
  if (evolutionType == 1) {
    y12 = 0.5*sqrt(yE*rNow);
    y23 = 0.5*sqrt(yE/rNow);
  } 
  else if (evolutionType == 2) {
    if (z < 0.5) {
      y12 = yE/2.;
      y23 = y12/rNow;
    } else {
      y23 = yE/2.;
      y12 = y23*rNow;
    }
  }
  else if (evolutionType == 3) {
    y12 = z*sqrt(yE);
    y23 = (1.-z)*sqrt(yE);
  } 
  else if (evolutionType == 4) {
    if (z <= 0.5) {
      y23 = pow2(yE)/2./pow2(z)
	*( pow2(1.-z)+sqrt(pow2(1.-z)*(1-2*z)) );
      y12 = y23*z/(1.-z);
    } else {
      // Re-use solution for z < 0.5 with z <-> 1-z => y12 <-> y23
      z=1.-z;
      y12 = pow2(yE)/2./pow2(z)
	*( pow2(1.-z)+sqrt(pow2(1.-z)*(1-2*z)) );
      y23 = y12*z/(1.-z);
    } 
  }
  // Use same as Type 1 as a catchall (should not occur)
  else {
    y12 = 0.5*sqrt(yE*rNow);
    y23 = 0.5*sqrt(yE/rNow);
  }

  // Check whether inside physical phase space
  if ( 1.-y12-y23 <= 0. || y12 >= 1. || y23 >= 1. || y12 <= 0. || y23 <= 0. ) {
    if (verbose >= 5) {
      printOut("::generateKinematics",
	       "=== Branching Vetoed. Outside physical Phase Space.");      
    }
    return false;
  }

  // Check hadronization veto 1: cutoff scale 
  bool hadAccept  = true;    
  if (evolutionType != cutoffType && cutoffType != 0) {
    double Q2forHad = RPtr->yE(y12,y23,cutoffType)*s;
    if (Q2forHad < pow2(cutoffScale)) {
      if (verbose >= 5) printOut("::generateKinematics",
	       "=== Branching Vetoed. In hadronization region.");            
      hadAccept = false;
    }
  }
  // Check hadronization veto 2: pion mass
  double mMin     = sqrt(RPtr->yE(y12,y23,2)*s/2);
  if (mMin < 1.01*particleDataPtr->m0(221) ) {
    if (verbose >= 5) printOut("::generateKinematics",
			       "=== Branching Vetoed. m < 1.01*mpi.");            
    hadAccept = false;
  }
  if ( ! hadAccept ) {
    trial->nFailed++;
    return false;
  }

  // Check for first-order matching (higher order ones done after full kin)
  bool doMatch = false;
  if (MEClass != 0 && nEmit+nSplit == 0) {
    doMatch = matchingPtr->isMatched(MEClass,nEmit,nSplit,false);
  }

  // Set flavors, kinematics maps, and physical antenna weights
  int kineMapType     =  1;
  trial->iAnt         = -1;
  double antPhys      = 0.0;
  double chargeFactor = 0.0;
  bool swapAB     = false;  
  if (branchType == 0) {
    // Gluon emission
    // XY -> XgY type
    trial->new1.id(trial->old1.id());
    trial->new2.id(21);
    trial->new3.id(trial->old2.id());
    if (trial->old1.id() == 21 && trial->old2.id() == 21) {
      // gg->ggg
      trial->iAnt  = iggEmit;
      antPhys     = getAnt(iggEmit)->a2(y12,y23);      
    } else if (trial->old1.id() == 21 || trial->old2.id() == 21) {
      // qg->qgg (and gqbar->ggqbar, but compute for qg->qgg in either case)
      trial->iAnt  = iqgEmit;
      antPhys      = getAnt(iqgEmit)->a2(y12,y23);      
      // ... put in massive modification here ...
      // Select between qg->qgg and gqbar->ggqbar
      if (trial->old1.id() == 21) swapAB = true;
    } else {
      // qqbar->qgqbar
      trial->iAnt  = iqqEmit;
      antPhys      = getAnt(iqqEmit)->a2(y12,y23);
      // ... put in massive modification here ...
    }
  } else {
    // Gluon splitting
    if (trial->old1.id() == 21 && trial->old2.id() == 21) {
      // gg->gqbarq (and gg->qbarqg)
      trial->iAnt  = iggSplit;
      antPhys      = getAnt(iggSplit)->a2(y12,y23);
      // Select between gg->gqbarq and gg->qbarqg
      if (rndmPtr->flat() < 0.5) {
	trial->new1.id(21);
	trial->new2.id(-trial->iNew);
	trial->new3.id(trial->iNew);
      } else {
	// Single swap = swap. Using gqbarq for qbarqg
	swapAB     = true;
	trial->new1.id(-trial->iNew);
	trial->new2.id(trial->iNew);
	trial->new3.id(21);
      }
    } else {
      // qg->qqbar'q' (and gqbar->qbar'q'q, but compute for qg->qqbar'q')
      trial->iAnt  = iqgSplit;
      antPhys      = getAnt(iqgSplit)->a2(y12,y23);
      // ... put in massive modification here ...
      // Select between  qg->qqbar'q' and gqbar->qbar'q'q 
      if (trial->old2.id() == 21) {
	trial->new1.id(trial->old1.id());
	trial->new2.id(-trial->iNew);
	trial->new3.id(trial->iNew);
      } else {
	// Single swap = swap. Using qg for gqbar without swapped invariants in
	// antPhys.
	swapAB     = true;
	trial->new1.id(-trial->iNew);
	trial->new2.id(trial->iNew);
	trial->new3.id(trial->old2.id());
      }      
    }

  }

  // If running in hyper-jet mode, subtract soft Eikonal for gluon emission
  // for emissions off the hard interaction
  if (hyperjet && branchType ==0 && trial->system == 0) {
    if (verbose >= 8) 
      printOut("generateKinematics","HyperJet mode: Subtracting Eikonal");
    antPhys -= 2*(1.-y12-y23)/(y12*y23);
  }

  // Verbose output
  if (verbose >= 8)
    cout<<" AntPhys  = "<<setprecision(6)<<antPhys/s<<endl;

  // Get charge factor and kinematics map
  chargeFactor = getAnt(trial->iAnt)->getChargeFactor();
  kineMapType = getAnt(trial->iAnt)->getKineMapType();
  
  // Reweight trial down to physical antenna by veto
  double wAntPhys  = chargeFactor * antPhys;
  double antTrial  = 2.0 * headRoomFactor / y12 / y23;    
  double wAntTrial = antTrial; 
  if (branchType == 0) {
    wAntTrial *= CA;
  } else {
    antTrial  *= 0.5; 
    wAntTrial *= TR*0.5;
  }
  Paccept[0] = wAntPhys/wAntTrial;

  // Verbose output
  if (verbose >= 8) {
    cout<<" AntTrial = "<<setprecision(6)<<antTrial/s<<endl;
    cout<<" RAhat    = "<<setprecision(6)<<antPhys/antTrial<<endl;
  }

  // Compute alternative reweight factors for uncertainty band
  if (uncertaintyBands) {
    for (int iSet=1; iSet<int(antennaSets.size()); iSet++) {
      double antAlt  = getAnt(trial->iAnt,iSet)->a2(y12,y23);
      double wAntAlt = antAlt * getAnt(trial->iAnt,iSet)->getChargeFactor();
      Paccept[iSet]  = wAntAlt/wAntPhys;
    }
  }

  // Reweight by alphaS if using pT as argument and not-pT ordering
  double pT2 = 0.25 * RPtr->Q2E(s,y12,y23,1);
  if (alphaSmode == 3 && evolutionType != 1) {
    double mu2 =  max(pow2(alphaSMuMin),pow2(alphaSscaleFactor/0.5) * pT2);
    Paccept[0] *= min(1.0,alphaS.alphaS(mu2)/alphaSMax);
  }

  // Partial NLL matching: impose alphaS(pT) to first order
  // Scale cancellation for uncertainty bands
  // (use alphaS(s) for scale canceling term, to be conservative)
  if (alphaSorder >= 1 && NLLMatchingLevel >= 0) {
    double mu2 = s;
    if (alphaSmode == 1) mu2 = RPtr->Q2E(s,y12,y23);
    else if (alphaSmode == 3) mu2 = pT2;
    mu2 *= pow2(alphaSscaleFactor);
    mu2 = max(mu2,pow2(alphaSMuMin));
    double mu = sqrt(mu2);
    // Use aphaS(sijk) for correction, thus making the correction
    // as small as possible, making the stabilization as conservative
    // as possible.
    double aS       = alphaS.alphaS(s);    
    double DeltaAS  = 0.0;
    // Target Scale: pT2 for gluon emission, mqq for qqbar
    // (this is before swap, so yqq = y23
    double mu2Stable = pT2;
    if (branchType != 0) mu2Stable = y23*s;
    // Match alphaS across thresholds    
    double Nf   = getNf(sqrt(mu2Stable));
    double NfMu = getNf(mu);
    bool accept = false;
    while (not accept) {
      double b0  = (11.0*NC - 2.0*NfMu)/(12.0*M_PI);
      if (NfMu == Nf) {
	DeltaAS   += aS * b0 * log(mu2/mu2Stable);
	accept = true;
      } else if (NfMu > Nf) {
	double Q2Low = pow2(getQMin(NfMu));
	DeltaAS     += aS * b0 * log(mu2/Q2Low);      
	// Update mu2 and Nf
	mu2   = Q2Low;
	NfMu -= 1;
      } else if (NfMu < Nf) {
	double Q2Hi  = pow2(getQMax(NfMu));
	DeltaAS     += aS * b0 * log(mu2/Q2Hi);      
	// Update mu2 and Nf
	mu2   = Q2Hi;
	NfMu += 1;	
      }
    }
    double NLLFactor = (1.0 + DeltaAS);
    // Restrict range to 0 < PNLLmu < 2 
    // (expansion otherwise divergent anyway)
    NLLFactor = min(2.0,NLLFactor);
    NLLFactor = max(0.0,NLLFactor);
    Paccept[0] *= NLLFactor;
    if (uncertaintyBands) {
      // Remember to cancel scale dependence of uncertainty tunes
      // individually?
      for (int iSet=1; iSet<int(antennaSets.size()); iSet++) {
	//	double antAlt = getAnt(trial->iAnt,iSet)->a2(y12,y23);
	Paccept[iSet] *= NLLFactor;
      }
    }
  }

  // Compute remaining invariant;
  double y13 = 1.0-y12-y23;
  // Swap if antenna invariants require it
  if (swapAB) {
    double tmp = y12;
    y12 = y23;
    y23 = tmp;
  }

  // Verbose output
  if (verbose >= 8) {
    printOut("generateKinematics","=> y12 = "+num2str(y12)+
	     "  y23 = "+num2str(y23)+"  y31 = "+num2str(y13));
  }

  // Compute invariants
  double s12 = y12*s;
  double s23 = y23*s;
  double s13 = y13*s;

  // Can check alternative hadronization vetos here

  if (verbose >= 7) {
    cout<<" (generateKinematics:) m  = "<<num2str(sqrt(s))
	<<"   m12 ="<<num2str(sqrt(s12))<<"   m23 ="<<num2str(sqrt(s23))
	<<"   m13 ="<<num2str(sqrt(s13))<<endl;
    RotBstMatrix M;
    M.toCMframe(trial->old1.p(), trial->old2.p());
    Vec4 p1cm=trial->old1.p();
    Vec4 p2cm=trial->old2.p();  
    p1cm.rotbst(M);
    p2cm.rotbst(M);
    cout <<" (generateKinematics:) starting dipole in CM\n";
    cout <<" p1cm = "<<p1cm<<" p2cm = "<<p2cm<<endl;
  }

  // Set up kinematics in rest frame.
  double sm1=trial->new1.m2();
  double sm2=trial->new2.m2();
  double sm3=trial->new3.m2();
  double x=2.0*sqrt(s);
  double E1=(s-s23+sm1)/x;
  double E2=(s-s13+sm2)/x;
  double E3=(s-s12+sm3)/x;
  double ap1=sqrt(pow2(E1)-sm1);
  double ap2=sqrt(pow2(E2)-sm2);
  double ap3=sqrt(pow2(E3)-sm3);
  double cos12=(2*E1*E2+sm1+sm2-s12)/2.0/ap1/ap2;
  double sin12=sqrt(abs(1.0-cos12*cos12));
  double cos13=(2*E1*E3+sm1+sm3-s13)/2.0/ap1/ap3;
  double sin13=sqrt(abs(1.0-cos13*cos13));
  Vec4 p1(0.0,0.0,ap1,E1);
  Vec4 p2(-ap2*sin12,0.0,ap2*cos12,E2);
  Vec4 p3(ap3*sin13,0.0,ap3*cos13,E3);

  // Verbose output
  if (verbose >= 8) {
    cout <<" (generateKinematics:) configuration in CM* (def: 1 along z)\n";
    cout <<" k1* =  "<<p1<<" k2* =  "<<p2<<" k3* =  "<<p3<<endl;
  }  

  // Choose azimuthal (phi) angle around dipole axis
  double phi = 2*M_PI*rndmPtr->flat();

  // Choose Global rotation around axis perpendicular to event plane

  // Default: psi ARIADNE-like
  double psi;
  if (kineMapType == 1) {
    // psi ARIADNE-like
    psi = E3*E3/(E1*E1+E3*E3)*(M_PI-acos(cos13));
  } else if (kineMapType == 2) {
    // psi PYTHIA-like. "Recoiler" remains along z-axis
    psi = 0.;
    if (s12 < s23 || (s12 == s23 && rndmPtr->flat() > 0.5) ) psi=M_PI-acos(cos13);
  } else {
    // Example of David's psi
    double f=y23/(y12+y23);
    double rho=sqrt(1.0+4.0*f*(1.0-f)*y12*y23/y13);
    double yaa=-((1.0-rho)*y13+2.0*f*y12*y23)/2.0/(1.0-y12);
    // Bug corrected for v 1.019
    //    psi=1.0+2.0*yaa/(1.0-y23);    
    psi=acos(1.0+2.0*yaa/(1.0-y23));    
  }    
    
  // Perform global rotations
  p1.rot(psi,phi);
  p2.rot(psi,phi);
  p3.rot(psi,phi);

  // verbose output
  if (verbose >= 7) {
    cout <<" (generateKinematics:) phi = "<<phi<<" psi = "<<psi<<endl;
    cout <<" (generateKinematics:) final momenta in CM\n";
    cout <<" k1cm = "<<p1<<" k2cm = "<<p2<<" k3cm = "<<p3<<endl;
  }  

  // Rotate and boost to LAB frame
  RotBstMatrix M;
  M.fromCMframe(trial->old1.p(),trial->old2.p());
  if (verbose >= 8) {
    cout <<" (generateKinematics:) boosting to LAB frame defined by\n";
    cout <<" p1 =   "<<trial->old1.p()<<" p2 =   "<<trial->old2.p()<<endl;}

  p1.rotbst(M);
  p2.rotbst(M);
  p3.rotbst(M);
  if (verbose >= 7) {
    cout <<" (generateKinematics:) final momenta in LAB\n";
    cout <<" k1 =   "<<p1<<" k2 =   "<<p2<<" k3 =   "<<p3<<endl;}

  // Save momenta
  trial->new1.p(p1);
  trial->new2.p(p2);
  trial->new3.p(p3);

  // Verbose output: check momentum mapping by explicit reclustering
  if (verbose >= 3) {
    vector<Vec4> pNew, pClu;
    pNew.push_back(trial->new1.p());
    pNew.push_back(trial->new2.p());
    pNew.push_back(trial->new3.p());
    vinCluPtr->vc3to2(kineMapType,pNew,0,1,2,pClu);
    double DeltaEP = sqrt(pow2(pClu[0].px()-trial->old1.px())
			  + pow2(pClu[0].py()-trial->old1.py())
			  + pow2(pClu[0].pz()-trial->old1.pz())
			  + pow2(pClu[0].e()-trial->old1.e())
			  + pow2(pClu[1].px()-trial->old2.px())
			  + pow2(pClu[1].py()-trial->old2.py())
			  + pow2(pClu[1].pz()-trial->old2.pz())
			  + pow2(pClu[1].e()-trial->old2.e()));
    DeltaEP /= pClu[0].e()+pClu[1].e();
    if (DeltaEP > TINY) {
      printErr("generateKinematics","relative deviation = "+num2str(DeltaEP)
	       +" in reclustering 3->2 kinematics");
    }
    if (verbose >= 7) {
      cout<<" Checking reclustering to "<<trial->iOld1<<"-"
	  <<trial->iOld2<<" using map "<<kineMapType<<endl;
      cout<<"      pahat = "<<num2str(pClu[0].px())<<" "<<num2str(pClu[0].py())
	  <<" "<<num2str(pClu[0].pz())<<" "<<num2str(pClu[0].e());
      cout<<"      pbhat = "<<num2str(pClu[1].px())<<" "<<num2str(pClu[1].py())
	  <<" "<<num2str(pClu[1].pz())<<" "<<num2str(pClu[1].e())<<endl;
      cout<<"      "<<setw(5)<<trial->iOld1<<" = "<<num2str(trial->old1.px())
	  <<" "<<num2str(trial->old1.py())
	  <<" "<<num2str(trial->old1.pz())
	  <<" "<<num2str(trial->old1.e());
      cout<<"      "<<setw(5)<<trial->iOld2<<" = "<<num2str(trial->old2.px())
	  <<" "<<num2str(trial->old2.py())
	  <<" "<<num2str(trial->old2.pz())
	  <<" "<<num2str(trial->old2.e())<<endl;
    }
  }
    
  return true;
}

//*********

// Return charge * a2() 
//    for a specific antenna = iAnt
//    with partons stored in p, indices a, r, and b.

double VinciaShower::a2(int iAnt, vector<Vec4> p, int a, int r, int b, 
			double charge) {

  if (iAnt < 0 ) return 0.0;
  double sar  = m2(p[a],p[r]);
  double srb  = m2(p[r],p[b]);	  
  double s    = m2(p[a],p[r],p[b]);
  if (s == 0.0) return 0.0;
  return charge * getAnt(iAnt)->a2(sar/s,srb/s)/s;
  
}

// Return a2() for default charge, otherwise as above

double VinciaShower::a2(int iAnt,vector<Vec4> p, int a,int r,int b) {
  if (iAnt < 0 ) return 0.0;
  double charge = getAnt(iAnt)->getChargeFactor();
  return a2(iAnt,p,a,r,b,charge);
}

//*********

/* Check whether radiation falls into sector
Important Note: not yet operational for gluon loops! (e.g., H->gg)
Inputs: 
   event : event record before branching
   trial : proposed trial branching
Output:
   sectorAccept : false if branching should be vetoed
*/

bool VinciaShower::sectorAccept(Event& event,Branchelemental* trial) {

  // Only do check if evolution mode is not global
  if ( ! sectorOrdering ) return true;

  // Check if this sector has smallest pT
  double pTSector = sqrt(RPtr->Q2E(trial->new1,trial->new2,trial->new3,1)/4.);
  double pTMin = mSystem/2.;
  for (int iDip = 0; iDip < int(branchelementals.size()); iDip++) {
    int ia = branchelementals[iDip].iOld1;
    int ir = branchelementals[iDip].iOld2;
    int ib = branchelementals[iDip].iR;
    // skip if no right-hand neighbor
    if (ib == 0) continue;
    // Compute pT for this triplet
    Vec4 pa = event[ia].p();
    Vec4 pr = event[ir].p();
    Vec4 pb = event[ib].p();
    // use new particles if neigboring pts affected by recoils
    if (ir == trial->iOld1) {
      pr = trial->new1.p();
      pb = trial->new2.p();
    } else if (ib == trial->iOld1) {
      pb = trial->new1.p();
    }
    if (ia == trial->iOld2) {
      pa = trial->new3.p();
    } else if (ir == trial->iOld2) {
      pa = trial->new2.p();
      pr = trial->new3.p();
    }
    // Keep track of smallest pT so far
    pTMin = min(pTMin,sqrt(RPtr->Q2E(pa,pr,pb,1)/4.));
    if (verbose >= 8) {
      printOut("sectorAccept","testing sector no "+num2str(iDip)+" pTMin = "
	       +num2str(pTMin)+" (pTsector = "+num2str(pTSector)+")");
    }
    if (pTSector > pTMin) return false;
  }

  // Return 
  return true;
  
}

//********* 

// Set event weight, check for non-unity and negative weights

  bool VinciaShower::setWeight(double weightIn, int iWeightIn) {

    if (! existsWeight(iWeightIn) ) return false;
    
    // For default / USR weight set, check for nonunity event weights.
    if ( iWeightIn == 0 ) {
      if (abs(abs(weightIn) - 1.0) > TINY && 
	  abs(abs(weightSave[iWeightIn])-1.0) < TINY) {
	nonunityWeightOccurred = true;     
	if (nNonunityWeight <= 5.0) 
	  if (verbose >= 1) 
	    printOut("VinciaShower::setWeight","Nonunity weight occurred, w = "+
		     num2str(weightIn)+((abs(nNonunityWeight-5.0) < TINY) ? 
					": further warnings suppressed" : " "));
      }
      
      // Check for negative weights 
      if (weightSave[iWeightIn] > 0.0 && weightIn < 0.0) {
	negativeWeightOccurred = true;
	if (nNegativeWeight <= 5)
	  if (verbose >= 1) 
	    printOut("VinciaShower::setWeight","Negative weight occurred, w = "+
		     num2str(weightIn)+((nNegativeWeight == 5) ? 
					": further warnings suppressed" : " "));
      } 

    }

    // Save new weight
    weightSave[iWeightIn] = weightIn;

    // Everything ok
    return true;
    
  }
  

//********* 

// OUTPUT ROUTINES

// Print header information
void VinciaShower::header() {
  cout<<setprecision(3);
  cout.setf(ios::left);
  cout << "\n";
  cout << " *-------  VINCIA"<<fixed<<setw(6)<<version
       <<" Global Initialization  ---------------------------------------*\n";
  cout << " | \n";
  cout << " | Vincia                           = "<<setw(12)<<
    ( (settingsPtr->flag("Vincia") ? "on" : "off")) <<"\n |\n";
  if (settingsPtr->flag("Vincia")) {
    cout << " | Matching :                       = ";
    if (matching && maxMatchTree >= 1) {
      cout <<setw(12)<< "on"<<"\n"; 
      cout <<" |               Strategy           = "<<setw(12)<<matchingStrategy<<"\n";
      cout <<" |               maxMatchTree       = "<<setw(12)<<maxMatchTree<<endl;
      cout <<" |               MGInterface        = "<<setw(12)
	   <<(isMGOn ? "on":"off")<<"\n";
    }
    else {
      cout<<setw(12)<<"off"<<"\n";
    }
    cout << " |\n";
    cout << " | Evolution:    Type               = "<<setw(12)
	 <<settingsPtr->mode("Vincia:evolutionType")<<"\n";
    cout << " |               orderingMode       = "<<setw(12)<<
      settingsPtr->mode("Vincia:orderingMode")<<"\n";
    cout << " |               nGluonToQuark      = "
	 << setw(12)<<nGluonToQuark<<"\n";
    cout << " |               QMinAll            = "
	 << setw(12)<<QMinAll<<"\n";
    if (sectorOrdering) {
      cout << " |               sectorOrdering     = "<<setw(12)<<
	( settingsPtr->flag("Vincia:sectorOrdering") ? "on":"off")<<"\n";
    }
    cout << " |\n"
	 << " | Cutoff:       Type               = "
	 << setw(12)<<settingsPtr->mode("Vincia:cutoffType")<<(settingsPtr->mode("Vincia:cutoffType") == -1 ? " (Pythia 8 pTevol)" :" ")<<"\n"    
	 << " |               Scale              = "
	 <<setw(12)<<scientific<<cutoffScale<<"\n |\n"
	 << " | Alpha_s:      alphaS(mZ)         = "
	 <<setw(12)<<scientific<<alphaSvalue<<"\n"
	 << " |               order              = "
	 <<setw(12)<<alphaSorder<<"\n";
    if (alphaSorder >= 1) {
      cout << " |               mode               = "
	   <<setw(12)<<settingsPtr->mode("Vincia:alphaSmode")<<"\n"
	   << " |               scaleFactor        = "
	   <<setw(12)<<scientific<<alphaSscaleFactor<<"\n"
	   << " |               NLLMatching        = "
	   <<setw(12)<<(( settingsPtr->mode("Vincia:NLLMatchingLevel") >= 0) ? "on":"off")<<"\n"
	   << " |               alphaSMax          = "
	   <<setw(12)<<num2str(alphaSMax)<<"\n |\n";
    }

    if ( settingsPtr->word("Vincia:antennaFile") != "none") {
      cout<<" | Antennae:     antennaFile        = "
	  <<settingsPtr->word("Vincia:antennaFile")<<"\n |\n"; 	
    }
    
    if (verbose >= 2) {
      for (unsigned int i=0;i<=4;i++) {
	cout.setf(ios::left);
	cout<<setprecision(2);
	cout<<" | * "<<getAnt(i)->getHumanName()
	    <<" ------------------------------------------------------------------------\n";
	cout<<" |   chargeFactor       = "<<fixed
	    <<setw(6)<<getAnt(i)->getChargeFactor()<<"\n";
	cout<<" |   useFiniteTerms     = "<<setw(6)<<
	  ( settingsPtr->flag("Vincia:useFiniteTerms") ? "on":"off")<<"\n";
	cout<<" |   kineMapType        = "
	    <<setw(6)<<getAnt(i)->getKineMapType()<<"\n";
	if (getAnt(i)->getInputMode() == 1) { 
	  int iprc = 2;
	  cout.setf(ios::right);
	  cout<<" |   Ant(ya,yb)         =  ";
	  bool linechange=false;
	  for (int a=-1; a <= 3; a++) {
	    if (a >= 0) linechange = true;
	    double Caa = getAnt(i)->getC(a,a);
	    if (Caa != 0.0) {
	      if (linechange) cout<<endl<<" |                         ";
	      linechange=false;
	      cout<<" ";
	      if (Caa < 0.0) cout<<"- "; 
	      else if (a >= 0) cout<<"+ ";
	      iprc = 2;
	      if ( int(abs(Caa) * 100 + 0.5) % 10 == 0) iprc = 1;
	      if (iprc == 1 && int(abs(Caa) * 10 +0.5) % 10 == 0) iprc=0;
	      if (a == -1 || Caa != 1.0) cout<<setprecision(iprc)<<abs(Caa);
	      if (a == -1) cout <<"/ya/yb";
	      else if (a == 1) cout <<"*ya*yb";
	      else if (a >= 2) cout <<"*ya^"<<a<<"*yb^"<<a;
	    }
	    for (int b=a+1; b <= 3; b++) {
	      double Cab = getAnt(i)->getC(a,b) ;
	      if (Cab != 0.0) {
		if (linechange) cout<<endl<<" |                         ";
		linechange=false;
		cout<<" ";
		if (Cab > 0.0) cout<<"+ ";
		else cout<<"- ";
		iprc=2;
		if ( int(abs(Cab) * 100 + 0.5) % 10 == 0) iprc = 1;
		if (iprc == 1 && int(abs(Cab) * 10 +0.5) % 10 == 0) iprc=0;
		cout<<setprecision(iprc)<<abs(Cab);
		if (a == 1) cout <<"*ya";
		else if (a >= 2) cout <<"*ya^"<<a;
		if (b == 1) cout <<"*yb";
		else if (b >= 2) cout <<"*yb^"<<b;
		if (a == -1) cout <<"/ya";
	      }
	      double Cba = getAnt(i)->getC(b,a);
	      if (Cba != 0.0) {
		if (linechange) cout<<endl<<" |                         ";
		linechange=false;
		cout<<" ";
		if (Cba > 0.0) cout<<"+ "; 
		else cout<<"- ";
		iprc=2;
		if ( int(abs(Cba) * 100 + 0.5) % 10 == 0) iprc = 1;
		if (iprc == 1 && int(abs(Cba) * 10 +0.5) % 10 == 0) iprc=0;
		cout<<setprecision(iprc)<<abs(Cba);
		if (a == 1) cout <<"*yb";
		else if (a >= 2) cout <<"*yb^"<<a;
		if (b == 1) cout <<"*ya";
		else if (b >= 2) cout <<"*ya^"<<b;
		if (a == -1) cout <<"/yb";
	      }
	    }
	  }
	  if (sectorOrdering) {
	    cout <<"\n |                        ";
	    for (int a=-1; a <= 3; a++) {
	      double Da = getAnt(i)->getD(a);
	      if (Da != 0.0) {
		if (Da > 0.0) cout<<"+ ";
		else cout<<"- ";
		iprc=2;
		if ( int(abs(Da) * 100 + 0.5) % 10 == 0) iprc = 1;
		if (iprc == 1 && int(abs(Da) * 10 +0.5) % 10 == 0) iprc=0;
		cout<<setprecision(iprc)<<abs(Da);
		if (a == -1) cout <<"/(1-ya)/yb";
		else if (a == 1) cout <<"*yb/(1-ya)";
		else if (a >= 2) cout <<"*yb^"<<a<<"/(1-ya)";
		cout<<" ";
	      }
	      double Ea = getAnt(i)->getE(a);
	      if (Ea != 0.0) {
		if (Ea > 0.0) cout<<"+ ";
		else cout<<"- ";
		iprc=2;
		if ( int(abs(Ea) * 100 + 0.5) % 10 == 0) iprc = 1;
		if (iprc == 1 && int(abs(Ea) * 10 +0.5) % 10 == 0) iprc=0;
		cout<<setprecision(iprc)<<abs(Ea);
		if (a == -1) cout <<"/(1-yb)/ya";
		else if (a == 1) cout <<"*ya/(1-yb)";
		else if (a >= 2) cout <<"*ya^"<<a<<"/(1-yb)";
		cout<<" ";
	      }
	    }
	  }
	  cout<<"\n |\n";
	} else {
	  // Enter alternative input modes here
	}
      }
    }
    cout <<     " |-------------------------------------------"
	 << "------------------------------------------\n |\n";
    cout << " | References :"<<endl;
    cout << " |    VINCIA   : Giele, Kosower, Skands, PRD78(2008)014026"<<endl;
    cout << " |    PYTHIA8  : Sjostrand, Mrenna, Skands, CPC178(2008)852"<<endl;
    if (matching && maxMatchTree >= 1 && isMGOn) {
      cout << " | The MADGRAPH interface relies on:"<<endl;
      cout << " |    HELAS    : Murayama et al., KEK-91-11 (1992)"<<endl;
      cout << " |    MADGRAPH : Alwall et al., JHEP09(2007)028"<<endl;
    }
  }
  cout << " |\n *-------  End VINCIA Global Initialization  "
       << "-----------------------------------------*\n\n";
  cout.setf(ios::right);
}  
  
// Print the list of dipoles.

void VinciaShower::list(ostream& os) {
  // Loop over dipole list and print it.
  for (int iDip = 0; iDip < int(branchelementals.size()); ++iDip) 
    if (branchelementals.size() == 1) branchelementals[iDip].list(os,true,true);
    else if ( iDip == 0 ) branchelementals[iDip].list(os,true,false);
    else if ( iDip == int(branchelementals.size())-1 ) branchelementals[iDip].list(os,false,true);
    else branchelementals[iDip].list(os);
}

void VinciaShower::printInfo(ostream& os) {    
  os << "\n";
  os << " *-------  VINCIA Statistics  -------------------------------------------------------------------------------------*\n";
  os << " |                                                                                                                 |\n"; 
  os << " | Matching was :                                              = ";
  if (matching && maxMatchTree >= 1) {
    os <<setw(9)<< "on"<<setw(40)<<" "<<" |\n"; 
    os <<" | matchingStrategy                                            = "
       <<setw(9)<<matchingStrategy<<setw(40)<<" "<<" |\n";
    os <<" | maxMatchTree                                                = "
       <<setw(9)<<maxMatchTree<<setw(40)<<" "<<" |\n";
    if (isMGOn) {
      os <<" | MGInterface                                                 = "
	 <<setw(9)<<(isMGOn ? "on":"off")<<setw(40)<<" "<<" |\n";
    }
    if (verbose >= 2) {
      os <<" | final headRoomFactor                                        = "<<setw(9)<<headRoomFactor<<setw(40)<<" "<<" |\n";
    }
  }
  else {
    cout<<setw(9)<<"off"<<setw(40)<<" "<<" |\n";
  } 
  os << " |                                                                        "
     <<setw(40)<<" "<<" |\n"; 
  os << " | Number of nonunity-weight events                            = "
     <<( (nNonunityWeight-nNegativeWeight <= TINY) ? "     none" 
	 : num2str(int(nNonunityWeight-nNegativeWeight),9) )<<setw(40)<<" "<<" |\n";
  os << " | Number of negative-weight events                            = "
     <<( (nNegativeWeight <= TINY) ? "     none" 
	 : num2str(int(nNegativeWeight),9)) <<setw(40)<<" "<<" |\n";
  os << " |                                                                        "
     <<setw(40)<<" "<<" |\n"; 
  os << " | Average event weight                                        = "
     << num2str(weightSumSave[0]/nWeights,9)<<" ";
  if (uncertaintyBands) {
    os << num2str(weightSumSave[1]/nWeights,9)<<" "
       << num2str(weightSumSave[2]/nWeights,9)<<" "
       << num2str(weightSumSave[3]/nWeights,9)<<" " ;
  } else {
    os << setw(39)<<" ";
  }
  os <<" |\n";
  os << " | Average event weight deviation from unity                   = "
     << num2str(weightSumSave[0]/nWeights-1)<<" ";
  if (uncertaintyBands) {  
    os << num2str(weightSumSave[1]/nWeights-1)<<" "
       << num2str(weightSumSave[2]/nWeights-1)<<" "
       << num2str(weightSumSave[3]/nWeights-1)<<" ";
  } else {
    os << setw(39)<<" ";
  }
  os <<" |\n";
  os << " | Scale factor to recover sum(w) = 1.0                        = "
     << num2str(nWeights/weightSumSave[0])<<" ";
  if (uncertaintyBands) {  
    os << num2str(nWeights/weightSumSave[1])<<" "
       << num2str(nWeights/weightSumSave[2])<<" "
       << num2str(nWeights/weightSumSave[3])<<" ";
  } else {
    os << setw(39)<<" ";
  }
  os << " |\n";
  os << " |                                                                        "
     <<setw(40)<<" "<<" |\n"; 
  /*
  os << " | Largest Paccept (occurred at QE = "<<num2str(reweightMaxPSaveQ)
     <<" )               = "<<num2str(reweightMaxPSaveP)<<"   |\n";
  if (reweightMaxPSaveQ > TINY && reweightMaxQSaveQ > TINY) { 
    os << " | Largest QE with Paccept > 1 ( P = "<<num2str(reweightMaxQSaveP)
       <<" )               = "<<num2str(reweightMaxQSaveQ)<<"   |\n";
       }
  os << " |                                                                           |\n"; 
  */
  os << " *-------  End VINCIA Statistics ----------------------------------------------------------------------------------*\n\n";
}

//**********************************************************************

// Branchelemental class member functions

//*********

// Simple print utility, showing the contents of the Branchelemental

void Branchelemental::list(ostream& os, bool header, bool footer) { 
  if (header) {
    // Header.
    os << "\n --------  VINCIA Dipole-Antenna Listing  -----------------------------"
       << "---------------------------------------------------------- \n \n" 
       << "  sys   col      antenna-type   L aunts   parents  R uncles"
       << "         m      Qold    saved trial type   Qbranch   O"
       << "rd   nFail\n"
       << fixed << setprecision(3);
  }
  os << setw(5) << system <<setw(6) << old1.col() ;
  os << setw(9) << old1.name() << setw(9) << old2.name();
  os << setw(5) << iLL << setw(5) << iL;
  os << setw(5) << iOld1 << setw(5) << iOld2;
  os << setw(5) << iR << setw(5) << iRR;
  os << num2str(m(),10) << num2str(Qold,10); 
  if (hasTrial && Qnew > 0.0) {
    if (iNew == 0) os << setw(20) << "gluon emission";
    else  os << setw(20) << "gluon splitting";
    os << num2str(Qnew,10);
    os << setw(6) << (unOrdering ? "n" : "y");
    os << num2str(nFailed,8);
  } else if (hasTrial) {
    os << num2str(0.0,30);
  } else {
    os << setw(30) << "-" <<setw(6) << (unOrdering ? "n" : "y");
    os << num2str(nFailed,8);
  }
  os << endl;   
  if (footer) {
  os << "\n --------  End VINCIA VinciaShower Dipole-Antenna Listing  ------------"
     << "----------------------------------------------------------\n";
  }
}

} // end namespace VINCIA
